package chat.control;

import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import java.net.*;

import RTIcap.*;             // The package containing our stubs.
import npacrti.*;            // The package containing our stubs.
import org.omg.CosNaming.*;  // 
import org.omg.CORBA.*;      // All CORBA applications need these classes.

import chat.*;
import chat.gui.*;
import chat.model.*;

/**
*  This demo is prepared to demostrate how you can use OWRTI package.
*  This is a Chat program based on interactions.
*  
*
* @date August 15, 1999 ( Updated August 15, 1999 )
* @author H. Timucin Ozdemir
*
<changelog>
 <date></date>
 <desc></desc>
 <date></date>
 <desc> </desc>
</changelog>
*/
public class ChatControlImpl implements ChatControlRTI {

   String  fedName = "Chatter1";
   String  fedCnfName = fedName+".fed";

   short   federateId;
   ORB     orb;

   String chatterTypeStr = new String("Chatter");
   String chatterMsgTypeStr = new String("Message");

   String SetNameTxt = new String("Name");
   String ExitTxt = new String("Exit");

   short  chatterTypeId = -1;
   int    chatterMsgTypeId = -1;

   java.lang.Object guard = new java.lang.Object();

   RTIcap.RTIambassador rtiAmb;

   boolean       advanceGranted = false;
   double        currentTime = 0.0; 
   double        timeStep = 5.0; 
   double        requestTime ; 

   /** Log File */
   PrintWriter myLog;

   short     myChatterId;

   boolean   isConnected = false;

   ChatView   view=null;
   ChatModel  model=null;

   /** */
   public ChatControlImpl( RTIcap.RTIambassador _rtiAmb, ORB _orb ) {
       rtiAmb = _rtiAmb;
       orb = _orb;
       try 
       { myLog = new PrintWriter(new FileWriter("chat.log"),true);
       } catch( java.io.IOException ioe) {}
   }// end of CONSTRUCTOR

   public void setView(ChatView _view)    { view=_view; }
   public void setModel(ChatModel _model) { model=_model; }

   /**
   *
   * Gets the RTTI (Meta-Object Protocol MOP) handles.
   *
   * Ask RTI about the class indentifier of each object, attribute, 
   * interaction and parameter. So that we can define, subscribe, publish,
   * and interpret these attributes/objects.
   *
   */
   boolean getRtiIds() {
      boolean ret = false;
      if( rtiAmb != null ) {

       try { chatterTypeId = rtiAmb.getInteractionClassHandle(chatterTypeStr); }
       catch( RTIcap.NameNotFound icnd ) {}
       catch( RTIcap.FederateNotExecutionMember icnd ) {}
       catch( RTIcap.RTIinternalError icnd ) {}

       try
       { chatterMsgTypeId = 
                  rtiAmb.getParameterHandle(chatterMsgTypeStr,chatterTypeId);
           ret = true;
       } catch( RTIcap.InteractionClassNotDefined icnd ) {}
         catch( RTIcap.NameNotFound icnd ) {}
         catch( RTIcap.FederateNotExecutionMember icnd ) {}
         catch( RTIcap.RTIinternalError icnd ) {}
      }// end of if
      return ret;
   }// end of getRtiIds()

   /**
   *
   * Describe our interest for publishing objects and attributes.
   * Describe our interest for receiving objects and attributes.
   * Describe our interest for publishing interations and parameters.
   * Describe our interest for receiving interations and parameters.
   *
   */
   boolean publishSubscribe() {
     boolean ret = false;
     if( rtiAmb != null ) {
       try
       { rtiAmb.subscribeInteractionClass( chatterTypeId, RTIcap._Boolean.RTI_TRUE);
       } catch( RTIcap.FederateLoggingServiceCalls  flsc) {}
         catch( RTIcap.FederateNotExecutionMember  fnem) {}
         catch( RTIcap.RTIinternalError  ie) {}
         catch( RTIcap.RestoreInProgress  rip) {}
         catch( RTIcap.InteractionClassNotDefined  icnd) {}
         catch( RTIcap.SaveInProgress  sip) {}

       try
       { rtiAmb.publishInteractionClass( chatterTypeId ); ret = true; } 
       catch( RTIcap.FederateNotExecutionMember fnem) {}
       catch( RTIcap.RestoreInProgress rip) {}
       catch( RTIcap.InteractionClassNotDefined icnd) {}
       catch( RTIcap.SaveInProgress sip) {}
       catch( RTIcap.RTIinternalError ie) {}

      }// end of if

      return ret;

   }// end of publishSubscribe()

   /**
   * 
   * We want to join the Federation but we do not know its ID and
   * whether it is on. Therefore, we first try to create this Federation.
   * If this federation is  already exists, we will receive the exception.
   * If not, it will be created for us.
   * Then we need to join this execution and get ourself a federate id.
   * 
   */
   boolean joinFederationExecution(RTIcap.FederateAmbassador fedAmb) {

      boolean ret = false;

      if( rtiAmb != null && model!=null && model.getChatterName()!=null ) {
        try { rtiAmb.createFederationExecution(fedName,fedCnfName); } 
        catch( RTIcap.FederationExecutionAlreadyExists icnd) {}
        catch( RTIcap.RTIinternalError ie) {}

        try
        { Date aDate = new Date();
          String aName = model.getChatterName()+aDate;
          federateId = rtiAmb.joinFederationExecution(aName.replace(' ','_'),fedName,fedAmb);
          ret = true;
        } catch( RTIcap.FederateAlreadyExecutionMember fnem) {}
          catch( RTIcap.FederationExecutionDoesNotExist icnd) {}
          catch( RTIcap.CouldNotOpenFED fnem) {}
          catch( RTIcap.ErrorReadingFED fnem) {}
          catch( RTIcap.SaveInProgress sip) {}
          catch( RTIcap.RestoreInProgress rip) {}
          catch( RTIcap.RTIinternalError ie) {}

      }// end of if

      return ret;

   }// end of joinFederationExecution() 

   /**
   *
   * Before we leave the Federation
   *  1. Delete all the object that we are suppose(promised) to simulate.
   *  2. resign from the federation.
   *  3. Try to destroy the federation execution. Note that if there 
   *     are still federates in the Federation, the federation execution
   *     won't be stopped. Only, we resign it. 
   *
   */
   void leaveFederationExecution() {
      if( rtiAmb != null ) {
         try 
         {
           rtiAmb.resignFederationExecution(RTIcap.ResignAction.DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);
         } catch( RTIcap.InvalidResignAction ie) {}
           catch( RTIcap.FederateOwnsAttributes ie) {}
           catch( RTIcap.FederateNotExecutionMember ie) {}
           catch( RTIcap.RTIinternalError ie) {}

        try { rtiAmb.destroyFederationExecution(fedName); } 
        catch( RTIcap.FederationExecutionDoesNotExist icnd) {}
        catch( RTIcap.FederatesCurrentlyJoined ie) {}
        catch( RTIcap.RTIinternalError ie) {}

      }// end of if

   }// end of leaveFederationExecution() 

   /** */
   public void resetTimeAdvanceGranted() { synchronized(guard) { advanceGranted = false; } }

   /**
   * This method will be invoked by ChatFedAmb object whenever
   *
   *   timeAdvanceGrant 
   *   timeConstrainedEnabled
   *   timeRegulationEnabled
   *
   * messages received.
   *
   */
   public void setTimeAdvanceGranted() { synchronized(guard) { advanceGranted = true; } }

   /** */
   public boolean isTimeAdvanceGranted() { return advanceGranted ; }

   /** */
   public void setCurrentTime( double _newTime) { 
     if( _newTime >= currentTime ) currentTime = _newTime;
   }// end of setCurrentTime()

   /** */
   public final void mySleep(long aa) {  }// end of mySleep()

   /**
   *
   * Ticks the RTIambassador. This will cause the delivery of waiting 
   * events in the Receive Order Queue.
   *
   */
   public void tickRTI( long sleepTime ) {
      while( !isTimeAdvanceGranted() ) {
         try { rtiAmb.tick( ); } 
         catch( RTIcap.RTIinternalError aa ) {}
         mySleep(2);
      }// end of while() 
   }// end of tickRTI()

   /** */
   public boolean isConnected() { return isConnected; }

   /**
   * Move the frame back, and destroy the execution.
   */
   public void doExit() {
     if( isConnected ) { leaveFederationExecution(); isConnected=false; }
   }// end of doExit()

   /**
   * Connects to the Federation Execution
   */
   void doRealyConnect()
   {
      int count = 0;

      myLog.println(" before ChatFedAmb definition ");

      /*******************************************************************/
      /** Define FederateAmbassador for Chat1. This objects serves  */
      /** as a callback mechanism so that RTI can bring us the necessary */ 
      /** events.                                                        */
      /*******************************************************************/
      ChatFedAmb fedAmb = new ChatFedAmb(this);
      org.omg.CORBA.ORB.init().connect(fedAmb);

      myLog.println(" ChatFedAmb is defined ");
      
      /************************************************************/
      /** Create FederationExecution, get RTI id of the objects,  */
      /** attributes, interactions, and parameters.               */
      /** Define interest for Subscription/Publishing.            */
      /************************************************************/
      if( joinFederationExecution(fedAmb) && getRtiIds() && publishSubscribe() )
      {
        
        /*********************************************************************/
        /** TimeConstrained : we want to receive timestamped events in order */
        /*********************************************************************/
        resetTimeAdvanceGranted();

        try
        { 
          rtiAmb.enableTimeConstrained( );
        } catch( RTIcap.TimeAdvanceAlreadyInProgress aa ) {}
          catch( RTIcap.EnableTimeConstrainedPending aa ) {}
          catch( RTIcap.TimeConstrainedAlreadyEnabled aa ) {}
          catch( RTIcap.FederateNotExecutionMember aa ) {}
          catch( RTIcap.SaveInProgress aa ) {}
          catch( RTIcap.RestoreInProgress aa ) {}
          catch( RTIcap.RTIinternalError aa ) {}

        /***************************************************/
        /** Tick RTI to get timeConstrainedEnabled message */
        /***************************************************/
        tickRTI(2);

        /**********************************************************/
        /** TimeRegulating : We want to vote for time advancement */
        /**********************************************************/
        resetTimeAdvanceGranted();

        try
        { 
          rtiAmb.enableTimeRegulation( currentTime, 5.0 );

        } catch( RTIcap.TimeAdvanceAlreadyInProgress aa ) {}
          catch( RTIcap.EnableTimeRegulationPending aa ) {}
          catch( RTIcap.ConcurrentAccessAttempted aa ) {}
          catch( RTIcap.TimeRegulationAlreadyEnabled aa ) {}
          catch( RTIcap.InvalidLookahead aa ) {}
          catch( RTIcap.InvalidFederationTime aa ) {}
          catch( RTIcap.FederateNotExecutionMember aa ) {}
          catch( RTIcap.SaveInProgress aa ) {}
          catch( RTIcap.RestoreInProgress aa ) {}
          catch( RTIcap.RTIinternalError aa ) {}

        /**************************************************/
        /** Tick RTI to get timeRegulationEnabled message */
        /**************************************************/
        tickRTI(2);

        try
        { 
           rtiAmb.enableAsynchronousDelivery( );
           isConnected = true;

        } catch( RTIcap.FederateNotExecutionMember aa ) {}
          catch( RTIcap.AsynchronousDeliveryAlreadyEnabled aa ) {}
          catch( RTIcap.ConcurrentAccessAttempted aa ) {}
          catch( RTIcap.SaveInProgress aa ) {}
          catch( RTIcap.RestoreInProgress aa ) {}
          catch( RTIcap.RTIinternalError aa ) {}

      }// end of if 

   }// end of doRealyConnect()

   /**
   * Connects for publishing.
   */
   public void doConnect() {
          if( isConnected ) 
          giveError(" You are *already* connected! ");
     else if( model==null || model.getChatterName()==null ) 
          giveError(" You did not specified your name connected! ");
     else doRealyConnect();
   }// end of doExit()

   /**
   * Disconnects from the Federation Execution.
   */
   public void doDisconnect() {
     if( !isConnected ) giveError(" You are *already* disconnected! ");
     else { leaveFederationExecution(); isConnected=false; }
   }// end of doDisconnect()

   /** */
   public void doMail(String msg) {
     if( !isConnected ) giveError(" You are disconnected! "); 
     else
      {
        if(msg!=null&&msg.length()>0)
        try
        { RTIcap.ParameterHandleValuePair vals[] = 
                                 new RTIcap.ParameterHandleValuePair[1];
           vals[0] = new RTIcap.ParameterHandleValuePair((short)chatterMsgTypeId,msg.getBytes());
           rtiAmb.sendInteraction(chatterTypeId,vals,"");
        } catch( RTIcap.InteractionClassNotDefined aa ) {}
          catch( RTIcap.InteractionClassNotPublished aa ) {}
          catch( RTIcap.InteractionParameterNotDefined aa ) {}
          catch( RTIcap.FederateNotExecutionMember aa ) {}
          catch( RTIcap.SaveInProgress aa ) {}
          catch( RTIcap.RestoreInProgress aa ) {}
          catch( RTIcap.RTIinternalError aa ) {}
         try { rtiAmb.tick( ); } 
         catch( RTIcap.RTIinternalError aa ) {}
      }
   }// end of doMail()

   /**
   * Displays the error message.
   * @param mess Error message.
   */
   void giveError(String mess) {
     if(view!=null) view.giveError(mess);
   }// end of giveError()

   /** */
   public short getChatterRTITypeId() { return (short) chatterTypeId; }

   /** */
   public short getChatterRTIMsgTypeId() { return (short) chatterMsgTypeId; }

   /**
   * @param receivedMsg
   */
   public void addMessage(byte[] rMsg ) {if(view!=null) view.addMessage(rMsg);}

}// end of class ChatControlImpl()

