package chat.gui;
import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import java.net.*;

import chat.control.*;
import chat.model.*;

/**
*  This demo is prepared to demostrate how you can use OWRTI package.
*  This is a Chat program based on interactions.
*  
* @date August 15, 1999 ( Updated August 15, 1999 )
* @author H. Timucin Ozdemir
*
<changelog>
 <date>6.30.1999</date>
 <desc>gif images are added to the user interface.</desc>
 <date>7.1.1999</date>
 <desc>Buttons are enabled based on the current state. 
       states: A: Not Connected and no federation name.
               B: Not Connected with federation name
               C: Connected
       state transitions:
          A->Name->B
          B->Connect->C
          C->Disconnect->B
 </desc>
</changelog>
*/
public class 
ChatViewImpl extends JFrame implements ActionListener, ChatView, ChatModel {

   String SetNameTxt = new String("Name");
   String ExitTxt = new String("Exit");

   JButton  connect;
   JButton  mail;
   JButton  disconnect;
   JButton  exit;
   JTextArea rcvMessages; 
   JTextField sendMessage; 

   String    myChatterName = null;

   JMenuItem setNameItem;

   JMenuItem exitItem;

   ChatControl control = null;

   /** */
   public ChatViewImpl( ChatControl _control ) {
       control = _control;
  
       JMenuBar menuBar = new JMenuBar();
       JMenu menu ;

       setJMenuBar(menuBar);
       
       //Build the first menu.
       menu = new JMenu("Menu");
       menuBar.add(menu);

       setNameItem = new JMenuItem(SetNameTxt);
       exitItem = new JMenuItem(ExitTxt);

       menu.add(setNameItem);
       menu.add(exitItem);
       setNameItem.addActionListener(this);
       exitItem.addActionListener(this);

       connect=defineJButton("Connect","gif/door02.gif","Connects to OWRTI",KeyEvent.VK_C,false);
       disconnect=defineJButton("Disconnect","gif/door01.gif","Disconnects from OWRTI",KeyEvent.VK_D,false);
       exit=defineJButton("Exit","gif/exit.gif","Stops the program",KeyEvent.VK_E,true);
       mail=defineJButton("Mail","gif/mail.gif","Sends the message",KeyEvent.VK_M,false);

       rcvMessages =  new JTextArea(5, 30);
       rcvMessages.setEditable(false);

       JPanel contentPane = new JPanel();
       contentPane.setLayout(new BorderLayout());
       JPanel nPanel = new JPanel();
       nPanel.add(connect); 
       nPanel.add(mail); 
       nPanel.add(disconnect); 
       nPanel.add(exit); 

       connect.addActionListener(this);
       mail.addActionListener(this);
       disconnect.addActionListener(this);
       exit.addActionListener(this);

       JPanel sPanel = new JPanel();
       sendMessage = new JTextField(30);
       sPanel.add(new JLabel("Message"));
       sPanel.add(sendMessage);
       sendMessage.addActionListener(this);

       contentPane.add(nPanel,BorderLayout.NORTH);       
       contentPane.add(rcvMessages,BorderLayout.CENTER);       
       contentPane.add(sPanel,BorderLayout.SOUTH);       

       setContentPane(contentPane);

   }// end of CONSTRUCTOR

   /** */ 
   public URL getCodeBase() { 
     URL url = null;
     try
     {  url = new URL("file:"+System.getProperty("user.dir")+System.getProperty("file.separator"));
     } catch( MalformedURLException mue) {}
     return url;
   }// end of getCodeBase()

   public String getChatterName() { return myChatterName; }

   /**
   * ActionListener interface to collect the user events.
   * @param e ActionEvent
   */
   public void actionPerformed(ActionEvent e) {
     if( e.getSource() instanceof JMenuItem ) {
        JMenuItem source = (JMenuItem)(e.getSource());
        System.out.println(" action :"+source.getText());
        if( source == exitItem ) doExit();
        else if( source == setNameItem ) doGetName();
     }
     else if( e.getSource() instanceof JButton ) {
            JButton source = (JButton)(e.getSource());
            System.out.println(" action :"+source.getText());

            if( source ==  exit ) doExit();
            else if( source ==  connect ) doConnect();
            else if( source ==  disconnect ) doDisconnect();
            else if( source ==  mail ) doMail();

          }
     else if( e.getSource() instanceof JTextField )
          {
            JTextField source = (JTextField)(e.getSource());
            System.out.println(" action :"+source.getText());
          }
   }// end of actionPerformed()

   /**
   * Move the frame back, and destroy the execution.
   */
   void doExit() {
     toBack(); 
     control.doExit();
     System.exit(0);
   }// end of doExit()

   /**
   * Connects for publishing.
   */
   void doConnect() {
          if( control.isConnected() ) 
          giveError(" You are *already* connected! ");
     else if( myChatterName==null ) 
          giveError(" You did not specified your name connected! ");
     else control.doConnect();
     if(control.isConnected()) {
        mail.setEnabled(true);
        disconnect.setEnabled(true);
        connect.setEnabled(false);
     }// end of if
   }// end of doConnect()

   /**
   * Disconnects from the Federation Execution.
   */
   void doDisconnect() {
     if(!control.isConnected()) giveError(" You are *already* disconnected! ");
     else control.doDisconnect();
     if(!control.isConnected()) {
       mail.setEnabled(false);
       disconnect.setEnabled(false);
       connect.setEnabled(true);
     }// end of if
   }// end of doDisconnect()

   /**
   */
   void doMail() {
     if( !control.isConnected() ) giveError(" You are disconnected! "); 
     else { control.doMail(myChatterName+" : "+sendMessage.getText());
            addMessage(myChatterName+" : "+sendMessage.getText(), Color.red);
            sendMessage.setText("");
          }
   }// end of doMail()

   /** */
   void doGetName() {
     GetName dialog = new GetName(this,myChatterName);
     dialog.pack();
     dialog.setLocationRelativeTo(this);
     dialog.show();
     myChatterName = dialog.getName();
     if( myChatterName!=null ) {
       if( myChatterName.trim().length()>0) {
         myChatterName=dialog.getName().trim();
         connect.setEnabled(true);
       } else myChatterName=null;
     }// end of if
   }// end of doGetName()

   JButton defineJButton(String bLabel, String iFile, 
                         String tip, int Mnemonic, boolean enabled) {
       ImageIcon theImage = null;
       try { theImage=new ImageIcon(new URL(getCodeBase()+iFile)); }
       catch(java.net.MalformedURLException mue) {}
       JButton ret    = new JButton(bLabel,theImage);
       ret.setToolTipText(tip);
       ret.setEnabled(enabled);
       ret.setMnemonic(Mnemonic);
       return ret;
   }// end of defineJButton()

   /**
   * Displays the error message in the warning window.
   * @param mess Error message.
   */
   public void giveError(String mess) {
     JOptionPane.showMessageDialog( this, "Sorry!"+"\n"+mess,"",
                                    JOptionPane.ERROR_MESSAGE);
   }// end of giveError()

   /**
   * @param receivedMsg
   */
   public void  addMessage(byte[] receivedMsg ) {
      String a = new String(receivedMsg);
      rcvMessages.append((a+"\n"));
   }// end of receivedMsg()

   /**
   * @param receivedMsg
   * @param txtColor
   */
   public void  addMessage(byte[] receivedMsg, Color txtColor ) {
      String a = new String(receivedMsg);
      addMessage(a,txtColor);
   }// end of receivedMsg()

   /**
   * @param receivedMsg
   * @param txtColor
   */
   public void  addMessage(String receivedMsg, Color txtColor ) {
      Color cColor = rcvMessages.getForeground();
      rcvMessages.setForeground(txtColor);
      rcvMessages.append((receivedMsg+"\n"));
      rcvMessages.setForeground(cColor);
   }// end of receivedMsg()

}// end of class ChatViewImpl

